(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-packages")
(document:envelop with-translation __ "packages-info-i18n")

(document:envelop with-attributes
                  (entry-pixmap entry-label entry-value
                                packages-tree-items packages-translation packages-tree-text))

(document:envelop with-container-presentations
                  ((status-entry '/packages/status-entry
                                 entry-pixmap
                                 entry-label
                                 entry-value)
                   (packages-tree '/packages/tree)))

(define *packages* (_ "Packages Installation"))

;-------------------------------------------------------------------------------
; convenience utils
;-------------------------------------------------------------------------------
(define (packages-pixmap fname)
  (string-append "packages/" fname ".png"))

(define (packages-with-splash thunk)
  (splash-message (_ "Please wait..."))
  (document:release)
  (thunk)
  (splash-message ""))

(define (packages-first-leaf lst)
  (let loop ((lst lst)
             (result '(0)))
    (if (pair? (cond-cadr lst))
        (loop (cadr lst) (append1 result 0))
        result)))

;-------------------------------------------------------------------------------
; transaction logging/replaying
;-------------------------------------------------------------------------------
(document:insert "/packages/log")

;-------------------------------------------------------------------------------
; view methods
;-------------------------------------------------------------------------------
(define *view-alist*
  (filter pair?
          (list
           ;;(last (document:insert "/packages/view-profile"))
           (last (document:insert "/packages/view-groups"))
           (last (document:insert "/packages/view-pending"))
           (last (document:insert "/packages/view-search")))))

(define *view* (make-cell (caar *view-alist*)))

(define (view->this)
  (assv-ref *view-alist* (cell-ref *view*)))

(define (view->label)
  (assv-ref (view->this) 'label))

(define (view->type)
  (assv-ref (view->this) 'type))

(define (view->sort)
  (assv-ref (view->this) 'sort))

(define (view->add major)
  ((assv-ref (view->this) 'add) (or major "")))

(define (view->major)
  ((assv-ref (view->this) 'major)))

(define (view->minor minor)
  ((assv-ref (view->this) 'minor) (or minor "")))

(define (elt-version elt)
  ((assv-ref (view->this) 'elt-version) (or elt "")))

(define (elt-summary elt)
  ((assv-ref (view->this) 'elt-summary) (or elt "")))

(define (elt-description elt)
  ((assv-ref (view->this) 'elt-description) (or elt "")))

(define (elt-state elt)
  (log-add ((assv-ref (view->this) 'elt-state) (or elt ""))))

;===============================================================================
; UI
;===============================================================================

;-------------------------------------------------------------------------------
; view selector
; note: major-update must be after search update to correct auto-translator setup
;-------------------------------------------------------------------------------
(define (view-update)
  (let ((view (view-id text)))
    (for-each
     (lambda (e)
       (and (string=? view (assv-ref (cdr e) 'label))
            (cell-set! *view* (car e))))
     *view-alist*))
  (search-update)
  (major-update))

(define (view-contents)
  (map (lambda (e) (assv-ref (cdr e) 'label)) *view-alist*))

(define (view-ui)
  (hbox
   (label (_ "View:"))
   (document:id
    view-id
    (combobox
     rows (view-contents)
     current 0
     (when selected (view-update))))))

;-------------------------------------------------------------------------------
; search
;-------------------------------------------------------------------------------
(define (packages-search)
  (view->add (search-keyword-id text))
  (major-update))

(define (search-update)
  (let ((under-search? (procedure? (assv-ref (view->this) 'add))))
    (major-id packages-translation (not under-search?))
    (search-id visibility under-search?)))

(define (search-ui)
  (document:id
   search-id
   (hbox
    visibility (procedure? (assv-ref (view->this) 'add))
    (label (_ "Keywords:"))
    (document:id
     search-keyword-id
     (edit "" (when return-pressed
                (packages-with-splash packages-search)))))))

;-------------------------------------------------------------------------------
; major
;-------------------------------------------------------------------------------
(define (major-contents)
  (rows->tree-items
   (map (lambda(x) (string-cut x #\|))
        (sort (view->major) (view->sort)))))

(define (major-update)
  (let ((contents (major-contents)))
    (major-id packages-tree-items contents)
    (and (pair? contents)
         (major-id current (packages-first-leaf contents))))
  (major-id selected))

(define (major-ui)
  (document:id
   major-id
   (packages-tree
    packages-translation #t
    (let ((contents (major-contents)))
      (list (packages-tree-items contents)
            (and (pair? contents)
                 (current (packages-first-leaf contents)))))
    (when selected
      (if  (or (null? (major-id tree-items))
               (tree-leaf? (major-id tree-items)
                           (major-id current)))
           (packages-with-splash minor-update))))))

;-------------------------------------------------------------------------------
; minor
;-------------------------------------------------------------------------------
(define (minor-contents)
  (let ((major (string-join (major-id packages-tree-text) "|")))
    (map
     (lambda (elt)
       (cons (car elt) (packages-pixmap (cdr elt))))
     (view->minor major))))

(define (minor-update)
  (let ((contents (minor-contents)))
    (minor-id rows contents)
    (and (pair? contents) (minor-id current 0)))
  (minor-id selected))

(define (minor-state)
  (elt-state (minor-id text))
  (minor-state-1))

(define (minor-state-1)
  (and (eqv? (view->type) 'dynamic)
       (major-id packages-tree-items (major-contents))
       (major-id current '(0)))
  (let ((idx (minor-id current)))
    (minor-id rows (minor-contents))
    (minor-id current idx))
  (status-update))

(define (minor-ui)
  (document:id
   minor-id
   (listbox
    (let ((contents (minor-contents)))
      (list rows contents
            (and (pair? contents) (current 0))))
    (when selected (info-update))
    (when return-pressed (packages-with-splash minor-state))
    (when double-clicked (packages-with-splash minor-state)))))

;-------------------------------------------------------------------------------
; info
;-------------------------------------------------------------------------------
(define (info-update)
  (let ((elt (minor-id text))
        (state (info-toggle-id state)))
    (if state
        (info-description-id text (__ (elt-description elt)))
        (info-summary-id text (__ (elt-summary elt))))))

(define (info-toggle)
  (let ((state (info-toggle-id state)))
    (info-summary-id visibility (not state))
    (info-description-id visibility state)
    (info-update)))

(define (info-toggle-ui)
  (document:id
   info-toggle-id
   (checkbox (_ "Show package description")
             (when toggled (info-toggle)))))

(define (info-ui)
  (hbox
   (document:id info-summary-id (edit "" alterability #f visibility #t))
   (document:id info-description-id (textbox "" alterability #f visibility #f))))

;-------------------------------------------------------------------------------
; status
;-------------------------------------------------------------------------------
(define (packages-status)
  (let ((reply (woo-read-first "/packages/control")))
    (map (lambda (e) (cons e (woo-get-option reply e)))
         '(available installed install upgrade remove keep dsize isize foobaz))))

(define (status-update)
  (let ((val-alist (packages-status)))
    (status-available-id entry-value (assv-ref val-alist 'available))
    (status-installed-id entry-value (assv-ref val-alist 'installed))
    (status-install-id entry-value (assv-ref val-alist 'install))
    (status-upgrade-id entry-value (assv-ref val-alist 'upgrade))
    (status-remove-id entry-value (assv-ref val-alist 'remove))
    (status-keep-id entry-value (assv-ref val-alist 'keep))
    (status-dsize-id entry-value (assv-ref val-alist 'dsize))
    (status-isize-id entry-value (assv-ref val-alist 'isize))

    (status-isize-id
     entry-label
     (or (and (string=? (assv-ref val-alist 'foobaz) "disk")
              (_ "Be freed on disk"))
         (_ "Be used on disk"))))

  (and (global 'frame:next)
       (frame:next activity (log-pending-empty?)))

  ;;(prev-button enabled (log-pending-empty?))

  (command-update-id activity (log-pending-empty?))
  (command-reset-id activity (not (log-pending-empty?)))
  (command-apply-id activity (not (log-pending-empty?)))
  (command-undo-id activity (not (log-undo-empty?)))
  (command-redo-id activity (not (log-redo-empty?))))

(define (status-toggle-ui)
  (document:id
   status-toggle-id
   (checkbox
    (_ "Show statistics")
    (when toggled
      (status-id visibility (status-toggle-id state))))))

(define (status-ui)
  (let ((val-alist (packages-status)))
    (document:id
     status-id
     (vbox visibility #f
           (document:id status-available-id
                        (status-entry (packages-pixmap "available")
                                      (_ "Available")
                                      (assv-ref val-alist 'available)))
           (document:id status-installed-id
                        (status-entry (packages-pixmap "installed")
                                      (_ "Installed")
                                      (assv-ref val-alist 'installed)))
           (document:id status-install-id
                        (status-entry (packages-pixmap "be-installed")
                                      (_ "install")
                                      (assv-ref val-alist 'install)))
           (document:id status-upgrade-id
                        (status-entry (packages-pixmap "be-upgraded")
                                      (_ "upgrade")
                                      (assv-ref val-alist 'upgrade)))
           (document:id status-keep-id
                        (status-entry (packages-pixmap "be-kept")
                                      (_ "keep")
                                      (assv-ref val-alist 'keep)))
           (document:id status-remove-id
                        (status-entry (packages-pixmap "be-removed")
                                      (_ "remove")
                                      (assv-ref val-alist 'remove)))
           (document:id status-dsize-id
                        (status-entry (packages-pixmap "moreinfo")
                                      (_ "Need to get")
                                      (assv-ref val-alist 'dsize)))
           (document:id status-isize-id
                        (status-entry (packages-pixmap "moreinfo")
                                      (_"Be used on disk")
                                      (assv-ref val-alist 'isize)))))))

;-------------------------------------------------------------------------------
; command toolbox
;-------------------------------------------------------------------------------
(define (command-popup subdoc)
  (let ((result (document:popup subdoc)))
    (or (and (pair? result)
             (document:popup '/packages/error 'error result))
        (and (not result)
             (command-reset))))
  (log-reset)
  (view-update)
  (status-update))

(define (command-reset)
  (woo-write "/packages/control" 'control "quit")
  (if (eqv? (view->type) 'dynamic)
      (major-update)
      (minor-update))
  (log-reset)
  (status-update))

(define (command-undo)
  (log-undo)
  (minor-state-1))

(define (command-redo)
  (log-redo)
  (minor-state-1))

(define (command-quit)
  (and (not (log-pending-empty?))
       (command-popup '/packages/install))
  (woo-write "/packages/control" 'control "quit"))

(define (command-ui)
  (let* ((values (packages-status))
         (pending
          (pair? (memq #f (map (lambda (e) (string=? "0" (assoc-ref values e)))
                               '(install upgrade remove))))))
    (groupbox
     ""
     orientation 'horizontal
     (document:id command-update-id
                  (button (_ "Update")
                          activity (not pending)
                          (when clicked (command-popup '/packages/update))))

     (document:id command-reset-id
                  (button (_ "Reset")
                          activity pending
                          (when clicked
                            (packages-with-splash command-reset))))

     (document:id command-undo-id
                  (button (_ "Undo")
                          activity (not (log-undo-empty?))
                          (when clicked
                            (packages-with-splash command-undo))))

     (document:id command-redo-id
                  (button (_ "Redo")
                          activity (not (log-redo-empty?))
                          (when clicked
                            (packages-with-splash command-redo))))

     (document:id command-apply-id
                  (button (_ "Apply")
                          activity pending
                          (when clicked (command-popup '/packages/install)))))))

;-------------------------------------------------------------------------------
; main
;-------------------------------------------------------------------------------
(groupbox
 ""
 (hbox
  (vbox
   (view-ui)
   (major-ui)
   (status-ui)
   (status-toggle-ui))
  (vbox
   (search-ui)
   (minor-ui)
   (info-ui)
   (info-toggle-ui))))

(command-ui)

(and (global 'frame:next-button)
     (frame:next activity (log-pending-empty?)))

(frame:on-next
 (lambda()
   (if (log-pending-empty?)
       (woo-write "/packages/control" 'control "quit")
       'cancel)))

;; (on-apply
;;  (command-quit))

(when loaded (info-update))

(document:root
 (when destroyed
   (woo-write "/packages/control" 'control "quit")))
