;------------------------------------------------------------------------------
; usage sample:
;(define *pkgs* (list "gdm"))
;
;(define (run-install packages)
;  (let ((error (document:popup '/packages/install
;                               'packages packages
;                               'quit-on-destroy #t)))
;    (or (and (pair? error)
;             (run-dialog `("/packages/error" error ,error)))
;      error)))
;
;(button "New" (when clicked (format #t "result=~S~%" (run-install *pkgs*))))
;------------------------------------------------------------------------------

(document:surround "/std/base")
(document:envelop with-translation _ "alterator-packages")

title (_ "Applying changes...")

(define *packages-progress-error* (make-cell '()))

(define (progress-handler args)
  (let ((command (car args))
        (arg (cadr args))
        (rest (if (pair? (cddr args)) (caddr args) "")))
    (cond
     ;; progress begin
     ((and (string=? command "progress")
           (string=? arg "begin"))
      (label-id text (_ "Preparing...")))

     ;; start of download/install/whatever
     ((and (string=? command "progress")
           (or (string=? arg "download")
               (string=? arg "install")))
      (label-id text (if (string=? arg "install") ( _"Install") (_ "Download")))
      (bar-id maximum (string->number rest))
      (bar-id value 0)
      (text-id text ""))

     ;; in progress
     ((string=? command "step")
      (let ((val (+ (string->number rest) (bar-id value))))
        (text-id append-text arg)
        (bar-id value val)))

     ;; error messages
     ((string=? command "error")
      (cell-set! *packages-progress-error*
                 (acons arg rest
                        (cell-ref *packages-progress-error*))))

     ;; normal progress end
     ((and (string=? command "progress")
           (string=? arg "end"))
      (document-exit (or (null? (cell-ref *packages-progress-error*))
                         (cell-ref *packages-progress-error*))))

     ;; progress abort -- sources weren't available
     ((and (string=? command "progress")
           (string=? arg "abort"))
      (document-exit #f))

     ;; not reached normally
     (else
      (text-id append-text (string-append "XXX" command ":" arg "\n"))))))

(define (packages-extract-label uri)
  (and uri
       (let ((from (string-index uri #\[))
             (to (string-index uri #\])))
         (and from to (substring uri (+ from 1) to)))))

(define (packages-install-ensure-sources)
  (let ((cdrom (packages-extract-label
                (find (lambda(elt) (string-prefix? "cdrom" elt))
                      (woo-list-names "/packages/sources/active")))))
    (or (not cdrom)
        (let ((current
               (woo-catch
                (lambda() (woo-get-option (woo-read-first "/packages/cdrom/ident") 'label))
                (lambda(reason) ""))))
          (or (string=? cdrom current)
              (and (eqv? 'ok
                         (document:popup-information
                          (string-append (_ "Please insert CDROM, labeled as ") cdrom) 'ok 'cancel))
                   (packages-install-ensure-sources)))))))

(define (packages-install-available packages)
  (filter
   (lambda (e)
     (string=? "available"
               (woo-catch
                (lambda ()
                  (woo-get-option
                   (woo-first-command
                    (woo-read (string-append "/packages/state/" e))) 'state))
                (lambda () ""))))
   packages))

(define (document-exit value)
  (remove-mailbox progress-handler)
  (document:end value)
  (simple-notify document:root 'action "retry"))

(define (packages-install)
  (if (packages-install-ensure-sources)
      (let* ((packages (global 'packages))
             (available
              (or (and (pair? packages)
                       (packages-install-available packages))
                  '())))
        (map
         (lambda (e)
           (woo-write
            (string-append "/packages/state/" e) 'state 'install))
         available)
        (if (and packages (null? available))
            (document:end #t)
            (begin (woo-write "/packages/control" 'control "commit")
                   (add-mailbox progress-handler))))
      (document:end #f)))

(document:id label-id (label (_ "Applying changes...")))
(document:id text-id (textbox "" alterability #f))
(document:id bar-id (progressbar maximum 1 value 0))

(document:root (when loaded (packages-install)))
