(document:envelop let () )

(define *packages-profiles-major-current* (make-cell #f))
(define *packages-profiles-minor-current* (make-cell #f))

(define *profiles* (woo-list-names "/packages_profile"))

(define (packages-profiles-major) *profiles*)

(define (packages-profiles-minor profile)
  (cell-set! *packages-profiles-major-current* profile)
  (cell-set! *packages-profiles-minor-current*
             (packages-profiles-minor-1 profile))
  (cell-ref *packages-profiles-minor-current*))

(define (packages-profiles-minor-1 profile)
  (let* ((profile-name (string-append "/packages_profile/" (->string profile)))
         (profile (woo-list profile-name)))
    (map
     (lambda (ele)
       (let ((name (woo-get-option ele 'name))
             (states (map (lambda(e)
                            (woo-get-option (woo-read-first (string-append "/packages/state/" e))
                                            'state))
                          (woo-get-option ele 'contents))))
         (cons name (cond ((member "available" states) "available")
                          ((member "be-installed" states) "be-installed")
                          (else "installed")))))
     profile)))

(define (packages-profiles-elt elt)
  (woo-read-first (string-append "/packages_profile/"
                                 (cell-ref *packages-profiles-major-current*)
                                 "/"
                                 elt)))

(define (packages-profiles-elt-description elt)
  (woo-get-option (packages-profiles-elt elt) 'description))

(define (packages-profiles-elt-contents elt)
  (woo-get-option (packages-profiles-elt elt) 'contents '() ))

(define (packages-profiles-elt-state elt)
  (let* ((state (assoc-ref (cell-ref *packages-profiles-minor-current*) elt))
         (command (cond ((or (string=? state "available")
                             (string=? state "upgradable")
                             (string=? state "be-removed")) "install")
                        ((or (string=? state "installed")
                             (string=? state "be-installed")) "remove")))
         (contents (packages-profiles-elt-contents elt))
         (trans (cons command contents)))
    (or (packages-log-undo-trans trans)
        (map
         (lambda (e)
           (woo-write (string-append "/packages/state/" e) 'state command))
         contents))
    trans))

(define (packages-profiles-elt-noop elt)
  (make-string 0))

;;-------------------------------------------------------------------------------
;; run install over all groups in profiles, where state eq 'required

(define (packages-profiles-list elt)
  (woo-list (string-append "/packages_profile/" elt)))

(define (packages-profiles-required? elt)
  (eq? (woo-get-option elt 'state) 'required))

(define (packages-extract-contents elt)
  (woo-get-option elt 'contents))

(define (packages-profiles-init)
  (packages-with-splash
   (lambda()
     (let ((to-install
            (fold (lambda (x y)
                    (lset-adjoin equal? y x))
                  '()
                  (map
                   packages-extract-contents
                   (filter
                    packages-profiles-required?
                    (append-map packages-profiles-list *profiles*))))))
       (for-each
        (lambda (l)
          (for-each
           (lambda (p)
             (woo-write
              (string-append "/packages/state/" p) 'state 'install)) l)
          (packages-log-add (cons "install" l)))
        to-install))))
  #f)

;;-------------------------------------------------------------------------------

(or (null? *profiles*)
    (and (global 'install-mode) (packages-profiles-init))
    (cons
     'profiles
     (list (cons 'label (_ "Profiles"))
           (cons 'type 'static)
           (cons 'major packages-profiles-major)
           (cons 'minor packages-profiles-minor)
           (cons 'elt-version packages-profiles-elt-noop)
           (cons 'elt-summary packages-profiles-elt-noop)
           (cons 'elt-description packages-profiles-elt-description)
           (cons 'elt-state packages-profiles-elt-state))))
